/*! The networking methods for our server components to call. In
particular, this file provides a listen() methods to handle creating
a server process so the other code doesn't need to care about how
these work. */

// This file is based on tokio's echo.rs example:
// https://github.com/tokio-rs/tokio/blob/master/examples/echo.rs

use tokio::io::{AsyncReadExt, AsyncWriteExt};
use tokio::net::TcpListener;

pub async fn listen(addr: String, fun: &mut dyn FnMut(Vec<u8>) -> Vec<u8>) {
    let listener = TcpListener::bind(&addr)
        .await
        .expect("Failed to create TcpListener");
    println!("Listening on: {}", addr);

    loop {
        // asynchronously wait for an inbound socket
        let (mut socket, _) = listener.accept().await.expect("Failed to create socket");

//        tokio::spawn(async move {
            loop {
                // get number of bytes to receive
                let mut nbuf: [u8; 4] = [0; 4];
                socket
                    .read(&mut nbuf)
                    .await
                    .expect("Failed to get number of bytes to read");
                let n = u32::from_be_bytes(nbuf);

                if n == 0 {
                    break;
//                    return;
                }

                let mut buf = vec![0; n.try_into().unwrap()];

                // receive data
                socket
                    .read(&mut buf)
                    .await
                    .expect("Failed to read data from socket");

                if n == 0 {
                    break;
                    // return;
                }

                let response = fun(buf);

                // send number of bytes in response
                let response_size = u32::to_be_bytes(response.len().try_into().unwrap());
                socket
                    .write_all(&response_size)
                    .await
                    .expect("Failed to write number of bytes to listen for");

                // send response
                socket
                    .write_all(&response)
                    .await
                    .expect("Failed to write data to socket");
            }
//        });
    }
}
